<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: xmlrpc.php 4346 2006-03-06 16:43:19Z andrew@m3.net $
*/
 
/**
 * The XMLRPC data access layer
 * This DAL makes a remote request to the origin server for the requested zone
 * It then returns the information retrieved to the deliver engine.
 *
 * It must also return the old cache in case of failure to connect to origin.
 *
 * @package    MaxDal
 * @subpackage DeliveryXMLRPC
 * @author     Chris Nutting <chris@m3.net>
 */

// Required files
require_once 'XML/RPC.php';

function getFromOrigin($function, $params)
{
    /**
    * @package    MaxDal
    * @subpackage Delivery
    * @author     Chris Nutting <chris@m3.net>
    *
    */
    
    $conf = $GLOBALS['_MAX']['CONF'];
    
    // If we know the id for this cache file, then touch the file so that concurrent processes don't spawn more XMLRPC calls
    if (!empty($GLOBALS['_MAX']['deliveryCacheId'])) {
        $deliveryCacheFilename = 'cache_' . md5('Cache_Lite_Function') . '_' . md5($GLOBALS['_MAX']['deliveryCacheId']);
        if (file_exists(MAX_CACHE . $deliveryCacheFilename)) {
            touch(MAX_CACHE . $deliveryCacheFilename);
        }
    }
    
    // Create an XML-RPC client to talk to the XML-RPC server
    $client = new XML_RPC_Client($conf['origin']['script'], $conf['origin']['host'], $conf['origin']['port']);
    $message = new XML_RPC_Message($function, array());
    
    // Add the parameters to the message
	$message->addParam(new XML_RPC_Value($params, $GLOBALS['XML_RPC_String']));
	
	 // Send the XML-RPC message to the server
    $response = $client->send($message, $conf['origin']['timeout'], $conf['origin']['protocol']);
	
    if ((!$response) || ($response->faultCode() != 0)) {
        if (defined('CACHE_LITE_FUNCTION_ERROR')) {
            return CACHE_LITE_FUNCTION_ERROR;
        } else {
            return null;
        }
    } else {
        // Decode the serialized response
        $value = $response->value();
        $value = $value->scalarval();
        $value = unserialize($value);
    }
    return $value;
}

function MAX_Dal_Delivery_getZoneLinkedAds($zoneId)
{
    return getFromOrigin('getZoneLinkedAds', $zoneId);
}

function MAX_Dal_Delivery_getLinkedAds($search)
{
    return getFromOrigin('getLinkedAds', $search);
}

function MAX_Dal_Delivery_getZoneInfo($zoneId)
{
    return getFromOrigin('getZoneInfo', $zoneId);
}

function MAX_Dal_Delivery_getAd($adId)
{
    return getFromOrigin('getAd', $adId);
}

function MAX_Dal_Delivery_getChannelLimiations($channelId)
{
    return getFromOrigin('MAX_Dal_Delivery_getChannelLimiations', $channelId);
}

function MAX_Dal_Delivery_pluginExecute($params)
{
    return getFromOrigin('pluginExecute', serialize($params));
}
?>
