<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: AllocateZoneImpressions.php 3416 2005-06-30 14:40:49Z andrew $
*/

require_once MAX_PATH . '/lib/max/Dal/Forecasting.php';
require_once MAX_PATH . '/lib/max/Maintenance/OperationInterval.php';
require_once MAX_PATH . '/lib/max/core/ServiceLocator.php';

require_once MAX_PATH . '/lib/max/Maintenance.php';
require_once 'Date.php';

require_once MAX_PATH . '/lib/max/Maintenance/Forecasting/Channel/ChannelLimitations.php';

/**
 * A class to find how many impressions/request/etc were in chosen interval in specific zone by channel.
 *
 * Note: Always do the multiplication before the division in the code, to try to 
 * avoid rounding errors.
 * 
 * @package    MaxMaintenance
 * @subpackage Forecasting
 * @author     Radek Maciaszek <radek@m3.net>
 */
class MAX_Maintenance_Forecasting_Channel_ChannelStatistic
{
    var $conf;
    var $oChannelLimitations;
    
    var $lastDateOI;
    var $upToDateOI;
    var $oUpdateToDate;
    var $oDateNow;
    
    /**
     * Containt mappings from configuration maintenance types to 
     * @var array
     */
    var $tableMapping = array(
        'requests'    => 'data_raw_ad_request',
        'impressions' => 'data_raw_ad_impression',
        'clicks'      => 'data_raw_ad_click',
    );
    
    /**
     * Containt mappings from configuration maintenance types to tables
     * @var array
     */
    var $_channelTables = array();
    
    /**
     * object of type MAX_Dal_Maintenance
     * @var object
     */
    var $oDal;
    
    /**
     * The constructor method.
     */
    function MAX_Maintenance_Forecasting_Channel_ChannelStatistic()
    {
        $this->_init();
    }
    
    /**
     * A private method to initialise the class. Set the configuration file
     * data, the current date, and details of when the Maintenance Statistics
     * and Maintenance Prioritisation engines last ran.
     * 
     * @access private
     */
    function _init()
    {
        $this->conf = $GLOBALS['_MAX']['CONF'];
        
        $this->oDal = & $this->_getDal();
        $this->oOperationInt = & $this->_getOperationIntUtils();
        
        $oServiceLocator = &ServiceLocator::instance();
        $this->oDateNow = &$oServiceLocator->get('now');
        if (!$this->oDateNow) {
            $this->oDateNow = &new Date();
        }
        $aDates = $this->oOperationInt->convertDateToOperationIntervalStartAndEndDates($this->oDateNow);
        $this->oUpdateToDate = $aDates['end'];
        $this->oChannelLimitations = new MAX_Maintenance_Forecasting_Channel_ChannelLimitations();
        
        $records = explode(',', $this->conf['maintenance']['channelForecasting']);
        if (!empty($records)) {
            foreach ($records as $record) {
                if (isset($this->tableMapping[$record])) {
                    $this->_channelTables[$record] = $this->tableMapping[$record];
                }
            }
        }
        
        // Prepare dates
        $this->lastDateOI = $this->oDal->getMaintenanceForecastingLastRunInfo();
        
        list($this->upToDateOI,) = Maintenance::convertDateToOperationIntervalStartAndEndDates($this->oDateNow);
        $this->upToDateOI->subtractSeconds(1);
    }
    
    /**
     * Method to create/register/return the Maintenance Data Access Layer.
     *
     * @access private
     */
    function &_getDal()
    {
        $oServiceLocator = &ServiceLocator::instance();
        $oDal = $oServiceLocator->get('MAX_Dal_Forecasting');
        if (!$oDal) {
            $oDal = &MAX_Dal_Forecasting::singleton();
            $oServiceLocator->register('MAX_Dal_Forecasting', $oDal);
        }
        return $oDal;
    }
    
    /**
     * Method to register an OperationInterval class.
     *
     * @access private
     */
    function &_getOperationIntUtils()
    {
        return OperationInterval::singleton();
    }
    
    /**
     * Using channel sql limitations run a query for every table and count number of records in zone
     *
     * @param array $zonesIds  Zones IDs
     * @param int $channelId   Channel ID
     */
    function summarizeZonesBySqlLimitations($zonesIds, $channelId)
    {
        $sqlLimitations = $this->oChannelLimitations->getSqlLimitations($zonesIds);
        
        foreach ($this->_channelTables as $identifier => $table) {
            $counter = $this->oDal->countNumberOfRecordsInZonesBySqlLimitations($sqlLimitations, $table);
            if($counter === false) {
                return false;
            }
            // Save this calculations in database
            $this->oDal->saveChannelSummaryForZones($channelId, $zonesIds, $identifier, $counter);
        }
    }

    /**
     * Using channel sql limitations run a query for every table and count number of records in zone
     *
     * @param int $zoneId     Zone ID
     * @param int $channelId  Channel ID
     */
    function summarizeZonesBySqlAndPhpLimitations($zonesIds, $channelId)
    {
        $sqlLimitations = $this->oChannelLimitations->getSqlLimitations($zonesIds);
        $phpLimitations = $this->oChannelLimitations->getPhpLimitations();
        
        foreach ($this->_channelTables as $identifier => $table) {
            $counter = $this->oDal->countNumberOfRecordsInZonesBySqlAndPhpLimitations($sqlLimitations, $phpLimitations, $table);
            if($counter === false) {
                return false;
            }
            // Save this calculations in database
            $this->oDal->saveChannelSummaryForZones($channelId, $zonesIds, $identifier, $counter);
        }
    }

    /**
     * Find how many channels impressions were in specific interval and zone
     * for channel limitations
     *
     * @param int $channelId  Channel ID
     * @param array $zoneIds  Zone IDs
     */
    function summarizeRawDataForChannel($channelId, $zoneIds)
    {
        $this->oChannelLimitations->buildLimitations($channelId);
        if (empty($this->oChannelLimitations->phpLimitations)) {
            // use only sql limitations so we could COUNT(*) all records
            $this->summarizeZonesBySqlLimitations($zoneIds, $channelId);
        } else {
            // get all records from database (using also sql limitations if exists)
            // run php limitations on them
            $this->summarizeZonesBySqlAndPhpLimitations($zoneIds, $channelId);
        }
    }

    /**
     * Using channel sql limitations run a query for every table and count number of records in zone
     *
     * @param Date $startForecastingDate  When forecasting started to work
     * @param Date $endForecastingDate    Maximum datTime till maintenance is allowed to run
     * @return bool Return false on any error
     */
    function summarizeRawData($startForecastingDate = null, $endForecastingDate = null)
    {
        $agenciesIds = $this->oDal->getAgenciesIds();
        foreach ($agenciesIds as $agencyId) {
            $channelIds = $this->oDal->getChannelsByAgencyId($agencyId);
            $publisherIds = $this->oDal->getPublishersIds($agencyId);
            foreach ($publisherIds as $publisherId) {
                $tmpChannelIds = $this->oDal->getChannelsIdsByAgencyPublisher($agencyId, $publisherId);
                // join agency channels with publisher channels
                $allChannelIds = $channelIds;
                foreach($tmpChannelIds as $channelId) {
                    if(!in_array($channelId, $allChannelIds)) {
                        $allChannelIds[] = $channelId;
                    }
                }
                $zoneIds = $this->oDal->getZonesIds($publisherId);
                // count number of impressions in chosen zones/channel
                foreach ($allChannelIds as $channelId) {
                    //MAX::debug('Maintenace forecasting - summarize data for channelId = '.$channelId);
                    $this->summarizeRawDataForChannel($channelId, $zoneIds);
                    if($endForecastingDate !== null) {
                        if(Date::compare($startForecastingDate, $endForecastingDate) > 1) {
                            MAX::raiseError('Channel Forecasting run more than '.
                                $this->conf['maintenance']['channelForecastingMaxRunTime']
                                .' minutes, exiting...');
                            return false;
                        }
                    }
                }
            }
        }
    }

    /**
     * The main method of the class, (could be run in the future by the controlling 
     * task runner class)
     *
     * @return int  Return number of summarizing loops or false on error
     */
    function run()
    {
        $this->_init();
        
        if(!is_a($this->upToDateOI, 'Date')) {
            MAX::raiseError('UpToDate date wasn\'t assign properly');
            return false;
        }
        if(!is_a($this->lastDateOI, 'Date')) {
            MAX::raiseError('LastDateOI date wasn\'t assign properly');
            return false;
        }
        $start = &new Date();
        $start->copy($this->lastDateOI);
        $start->addSeconds(1);
        
        $count = 0;
        
        if($this->conf['maintenance']['channelForecastingMaxRunTime'] > 0) {
            $startForecastingDate = new Date();
            $endForecastingDate   = new Date($startForecastingDate);
            $endForecastingDate->addSeconds($this->conf['maintenance']['channelForecastingMaxRunTime'] * 60);
        } else {
            $startForecastingDate = null;
            $endForecastingDate   = null;
        }
        
        while (Date::compare($start, $this->upToDateOI) < 0) {
            // Calculate the end of the operation interval
            $end = &new Date();
            $end->copy($start);
            $end->addSeconds(($this->conf['maintenance']['operationInterval'] * 60) - 1); // Plus one OI
            
            $startForecastingTaskDate = new Date();
            if($this->oDal->setIntervalDates($start, $end)) {
                MAX::debug('Maintenace forecasting - summarize data for interval: '.$end->format('%Y-%m-%d %H:%M:%S'));
                // Summarize records in channels/zones
                $ret = $this->summarizeRawData($startForecastingDate, $endForecastingDate);
                if($ret === false) {
                    return $count;
                }
                
                // Save last run info
                $this->oDal->setMaintenanceForecastingLastRunInfo($startForecastingTaskDate, new Date(), $end);
            } else {
                return false;
            }
            
            // Go to the next operation interval
            $start->addSeconds($this->conf['maintenance']['operationInterval'] * 60);
            
            $count++;
        }
        
        return $count;
    }

}

?>