<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: statsFields.php 4761 2006-04-28 13:28:15Z matteo@beccati.com $
*/

require_once MAX_PATH . '/lib/max/Plugin/Common.php';
require_once MAX_PATH . '/lib/max/Plugin/Translation.php';

/**
 * Plugins_StatsFields is an abstract class for every Stats field plugin.
 *
 * @package    MaxPlugins
 * @subpackage Plugins_StatsFields
 * @author     Matteo Beccati <matteo@beccati.com>
 * @abstract
 */
class Plugins_StatsFields extends MAX_Plugin_Common
{
    /**
     * @var array
     */
    var $_fields;
    var $_prefs;
    
    /**
     * Return name of plugin
     *
     * @abstract
     * @return string A string describing the class.
     */
    function getName()
    {
        Max::debug('Cannot run abstract method');
        exit();
    }
    
    function getFields()
    {
        $fields = array();
        foreach ($this->_fields as $k => $v) {
            $fields[$k] = $v['name'];
        }
        
        return $fields;
    }
    
    function getEmptyRow()
    {
        $names = array();
        foreach (array_keys($this->_fields) as $k) {
            $names[$k] = 0;
        }
        
        return $names;
    }
    
    function getColumnLinks()
    {
        $links = array();
        foreach ($this->_fields as $k => $v) {
            if (!empty($v['link'])) {
                $links[$k] = $v['link'];
            }
        }
        
        return $links;
    }
    
    function getSumFieldNames()
    {
        $fields = array();
        foreach ($this->_fields as $k => $v) {
            if ($v['format'] != 'percent') {
                $fields[] = $k;
            }
        }
        
        return $fields;
    }
    
    function getVisibleColumns()
    {
        // Get the preferences
        $pref = $GLOBALS['_MAX']['PREF'];

        $columns = array();
        foreach (array_keys($this->_fields) as $k) {
            if (isset($this->_prefs[$k])) {
                $var = $this->_prefs[$k]['var'];
                $columns[$k] = isset($pref[$var]) && phpAds_isUser($pref[$var]);
            } else {
                $columns[$k] = true;
            }
        }
        
        return $columns;
    }
    
    function getVisibilitySettings()
    {
        // Get the preferences
        $pref = $GLOBALS['_MAX']['PREF'];

        $prefs = array();
        foreach (array_keys($this->_fields) as $k) {
            if (isset($this->_prefs[$k])) {
                $var = $this->_prefs[$k]['var'];
                $prefs[$var] = $this->_prefs[$k]['name'];
            }
        }
        
        return $prefs;
    }
    
    function addQueryParams(&$aParams)
    {
    }

    function mergeData(&$aRows, $method, $aParams)
    {
    }
        
    /**
     * Add the fields which require calculations
     *
     * @param array Row of stats
     */
    function summarizeStats(&$row)
    {
        Max::debug('Cannot run abstract method');
        exit();
    }

    /**
     * Calculate averages
     *
     * @param array Total stats
     * @param array Number of entries
     * @return array Averages array
     */
    function summarizeAverage($total, $count)
    {
        $average = array();
        foreach ($this->_fields as $k => $v) {
            $average[$k] = $count ? $total[$k] / $count : 0;
        }
        
        return $average;
    }
    
    /**
     * Format a row of stats according to the user preferences
     *
     * @param array Row of stats
     */
    function formatStats(&$row)
    {
        foreach ($this->_fields as $k => $v) {
            if (isset($row[$k])) {
                if ($v['format'] == 'percent') {
                    $row[$k] = phpAds_formatPercentage($row[$k]);
                 } elseif ($v['format'] == 'currency') {
                    $row[$k] = phpAds_formatNumber($row[$k], 2);
                } else {
                    $row[$k] = phpAds_formatNumber($row[$k]);
                }
            }
        }
    }
}

?>
